# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class StopRun(BaseModel):
    stop_run: Optional["StopRunStopRun"] = Field(alias="stopRun")


class StopRunStopRun(BaseModel):
    success: Optional[bool]


StopRun.model_rebuild()
