# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class UpdateCustomMetricResult(BaseModel):
    update_custom_metric_result: Optional[
        "UpdateCustomMetricResultUpdateCustomMetricResult"
    ] = Field(alias="updateCustomMetricResult")


class UpdateCustomMetricResultUpdateCustomMetricResult(BaseModel):
    id: str


UpdateCustomMetricResult.model_rebuild()
