# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class SetArchivedStatusCustomMetrics(BaseModel):
    set_archived_status_custom_metrics: Optional[
        "SetArchivedStatusCustomMetricsSetArchivedStatusCustomMetrics"
    ] = Field(alias="setArchivedStatusCustomMetrics")


class SetArchivedStatusCustomMetricsSetArchivedStatusCustomMetrics(BaseModel):
    affected_ids: Optional[List[Optional[str]]] = Field(alias="affectedIds")


SetArchivedStatusCustomMetrics.model_rebuild()
