# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class UpsertCustomMetric(BaseModel):
    upsert_custom_metric: Optional["UpsertCustomMetricUpsertCustomMetric"] = Field(
        alias="upsertCustomMetric"
    )


class UpsertCustomMetricUpsertCustomMetric(BaseModel):
    metric: "UpsertCustomMetricUpsertCustomMetricMetric"


class UpsertCustomMetricUpsertCustomMetricMetric(BaseModel):
    id: Any


UpsertCustomMetric.model_rebuild()
UpsertCustomMetricUpsertCustomMetric.model_rebuild()
