# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class SetCustomMetrics(BaseModel):
    set_custom_metrics: Optional["SetCustomMetricsSetCustomMetrics"] = Field(
        alias="setCustomMetrics"
    )


class SetCustomMetricsSetCustomMetrics(BaseModel):
    success: bool


SetCustomMetrics.model_rebuild()
