# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class UploadLocalEvaluation(BaseModel):
    upload_local_evaluation: Optional["UploadLocalEvaluationUploadLocalEvaluation"] = (
        Field(alias="uploadLocalEvaluation")
    )


class UploadLocalEvaluationUploadLocalEvaluation(BaseModel):
    uploads: List["UploadLocalEvaluationUploadLocalEvaluationUploads"]


class UploadLocalEvaluationUploadLocalEvaluationUploads(BaseModel):
    id: str
    score: float
    feedback: str
    created_at: datetime = Field(alias="createdAt")


UploadLocalEvaluation.model_rebuild()
UploadLocalEvaluationUploadLocalEvaluation.model_rebuild()
