# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import RunStatus


class UpdateRunStatus(BaseModel):
    update_run_status: Optional["UpdateRunStatusUpdateRunStatus"] = Field(
        alias="updateRunStatus"
    )


class UpdateRunStatusUpdateRunStatus(BaseModel):
    run: Optional["UpdateRunStatusUpdateRunStatusRun"]


class UpdateRunStatusUpdateRunStatusRun(BaseModel):
    run_id: str = Field(alias="runId")
    status: RunStatus


UpdateRunStatus.model_rebuild()
UpdateRunStatusUpdateRunStatus.model_rebuild()
