query listProjects($offset: Int!, $limit: Int!, $search: String!) {
  projectsWithCount(
    filterOptions: {
      offset: $offset
      limit: $limit
      archived: false
      search: $search
    }
  ) {
    projects {
      id
      name
      slug
      created
      isDefault
    }
    count
  }
}
