# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import List

from pydantic import Field

from .base_model import BaseModel
from .fragments import CheckOutputTypeFields


class GetTestSuiteData(BaseModel):
    test_suite: "GetTestSuiteDataTestSuite" = Field(alias="testSuite")


class GetTestSuiteDataTestSuite(BaseModel):
    description: str
    id: str
    project: "GetTestSuiteDataTestSuiteProject"
    org: str
    title: str
    created: datetime
    global_checks: List["GetTestSuiteDataTestSuiteGlobalChecks"] = Field(
        alias="globalChecks"
    )


class GetTestSuiteDataTestSuiteProject(BaseModel):
    slug: str


class GetTestSuiteDataTestSuiteGlobalChecks(CheckOutputTypeFields):
    pass


GetTestSuiteData.model_rebuild()
GetTestSuiteDataTestSuite.model_rebuild()
