# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .fragments import TestFragment


class UnfinishedTests(BaseModel):
    unfinished_tests: "UnfinishedTestsUnfinishedTests" = Field(alias="unfinishedTests")


class UnfinishedTestsUnfinishedTests(BaseModel):
    unfinished_tests: List["UnfinishedTestsUnfinishedTestsUnfinishedTests"] = Field(
        alias="unfinishedTests"
    )
    test_suite_size: int = Field(alias="testSuiteSize")


class UnfinishedTestsUnfinishedTestsUnfinishedTests(TestFragment):
    pass


UnfinishedTests.model_rebuild()
UnfinishedTestsUnfinishedTests.model_rebuild()
