# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class StartRun(BaseModel):
    start_run: Optional["StartRunStartRun"] = Field(alias="startRun")


class StartRunStartRun(BaseModel):
    run_id: str = Field(alias="runId")


StartRun.model_rebuild()
