# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class StartCustomMetricTask(BaseModel):
    start_custom_metric_task: Optional["StartCustomMetricTaskStartCustomMetricTask"] = (
        Field(alias="startCustomMetricTask")
    )


class StartCustomMetricTaskStartCustomMetricTask(BaseModel):
    message_id: Optional[str] = Field(alias="messageId")


StartCustomMetricTask.model_rebuild()
