fragment TestFragment on TestType {
  id
  inputUnderTest
  context
  tags
  crossVersionId
  goldenOutput
  fileIds
  checks {
    operator
    criteria
    modifiers {
      optional
      severity
      examples {
        type
        text
      }
      extractor
      conditional {
        operator
        criteria
      }
      category
      displayMetrics
    }
  }
  testSuite {
    id
  }
}
