#import TestResultFields, ResultJsonElementTypeFields from "../fragments/shared.graphql"

query SingleTestResultReviewsWithCount(
  $runId: String!
  $filterOptions: TestReviewFilterOptionsInput
) {
  testResultReviewsWithCount(runId: $runId, filterOptions: $filterOptions) {
    count
    singleTestResults {
      reviewedBy
      ...TestResultFields
      singleTestReviews {
        id
        completedBy
        feedback
        completedAt
        startedAt
        createdBy
        status
        perCheckTestReview {
          binaryHumanEval
          isFlagged
        }
        testResult {
          id
          resultJson {
            ...ResultJsonElementTypeFields
          }
        }
        customReviewValues {
          template {
            id
            name
            instructions
            optional
            categories
            type
            minValue
            maxValue
          }
          value
        }
      }
    }
  }
}
