# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class GetCustomMetric(BaseModel):
    custom_metric: "GetCustomMetricCustomMetric" = Field(alias="customMetric")


class GetCustomMetricCustomMetric(BaseModel):
    id: Any
    project: "GetCustomMetricCustomMetricProject"
    archived: bool
    name: str
    description: Optional[str]
    file_id: str = Field(alias="fileId")


class GetCustomMetricCustomMetricProject(BaseModel):
    slug: str


GetCustomMetric.model_rebuild()
GetCustomMetricCustomMetric.model_rebuild()
