# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class BatchAddQuestionAnswerPairs(BaseModel):
    batch_add_question_answer_pairs: Optional[
        "BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairs"
    ] = Field(alias="batchAddQuestionAnswerPairs")


class BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairs(BaseModel):
    question_answer_pairs: List[
        "BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairs"
    ] = Field(alias="questionAnswerPairs")


class BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairs(
    BaseModel
):
    id: Any
    input_under_test: str = Field(alias="inputUnderTest")
    llm_output: str = Field(alias="llmOutput")
    file_ids: List[str] = Field(alias="fileIds")
    context: Any
    output_context: Any = Field(alias="outputContext")
    test: (
        "BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairsTest"
    )


class BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairsTest(
    BaseModel
):
    id: str


BatchAddQuestionAnswerPairs.model_rebuild()
BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairs.model_rebuild()
BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairs.model_rebuild()
