# Generated by ariadne-codegen
# Source: vals/graphql/

from .base_model import BaseModel
from .enums import RunStatus


class GetRunStatus(BaseModel):
    run: "GetRunStatusRun"


class GetRunStatusRun(BaseModel):
    status: RunStatus


GetRunStatus.model_rebuild()
