# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class GetTestSuitesWithCount(BaseModel):
    test_suites_with_count: "GetTestSuitesWithCountTestSuitesWithCount" = Field(
        alias="testSuitesWithCount"
    )


class GetTestSuitesWithCountTestSuitesWithCount(BaseModel):
    test_suites: List["GetTestSuitesWithCountTestSuitesWithCountTestSuites"] = Field(
        alias="testSuites"
    )


class GetTestSuitesWithCountTestSuitesWithCountTestSuites(BaseModel):
    id: str
    title: str
    description: str
    created: datetime
    creator: str
    last_modified_by: str = Field(alias="lastModifiedBy")
    last_modified_at: datetime = Field(alias="lastModifiedAt")
    folder: Optional["GetTestSuitesWithCountTestSuitesWithCountTestSuitesFolder"]


class GetTestSuitesWithCountTestSuitesWithCountTestSuitesFolder(BaseModel):
    id: Any
    name: str


GetTestSuitesWithCount.model_rebuild()
GetTestSuitesWithCountTestSuitesWithCount.model_rebuild()
GetTestSuitesWithCountTestSuitesWithCountTestSuites.model_rebuild()
