# Generated by ariadne-codegen
# Source: http://localhost:8000/graphql/

from enum import Enum


class ChartMetric(str, Enum):
    passRate = "passRate"
    successRate = "successRate"
    refusalRate = "refusalRate"
    tokenUsage = "tokenUsage"
    lowConfidenceChecks = "lowConfidenceChecks"


class ChartType(str, Enum):
    line = "line"
    bar = "bar"
    table = "table"


class DropdownSectionEnum(str, Enum):
    CONSTITUTIONAL = "CONSTITUTIONAL"
    SEMANTIC = "SEMANTIC"
    STRING_MATCHING = "STRING_MATCHING"
    FORMATTING = "FORMATTING"
    CONSISTENT_WITH_SOURCE = "CONSISTENT_WITH_SOURCE"
    OTHER = "OTHER"


class EvaluationStatus(str, Enum):
    IN_PROGRESS = "IN_PROGRESS"
    ERROR = "ERROR"
    SUCCESS = "SUCCESS"


class AppQuestionAnswerSetCreationMethodChoices(str, Enum):
    LIVE_QUERY = "LIVE_QUERY"
    SDK = "SDK"
    UPLOAD = "UPLOAD"


class RunStatus(str, Enum):
    IN_PROGRESS = "IN_PROGRESS"
    ERROR = "ERROR"
    SUCCESS = "SUCCESS"
    PAUSE = "PAUSE"
    PENDING = "PENDING"
    AWAITING_BATCH = "AWAITING_BATCH"
    CANCELLED = "CANCELLED"
    PAUSING = "PAUSING"


class TestResultReviewStatusEnum(str, Enum):
    PENDING = "PENDING"
    COMPLETED = "COMPLETED"


class RunReviewStatusEnum(str, Enum):
    PENDING = "PENDING"
    COMPLETED = "COMPLETED"
    ARCHIVED = "ARCHIVED"


class TemplateType(str, Enum):
    CATEGORICAL = "CATEGORICAL"
    NUMERICAL = "NUMERICAL"
    FREE_TEXT = "FREE_TEXT"


class AppPairwiseTestResultReviewWinningRunChoices(str, Enum):
    A = "A"
    B = "B"
    TIE = "TIE"


class WinningRunEnum(str, Enum):
    A = "A"
    B = "B"
    TIE = "TIE"


class TierEnum(str, Enum):
    FREE = "FREE"
    STARTUP = "STARTUP"
    ENTERPRISE = "ENTERPRISE"


class DateFilterEnum(str, Enum):
    NONE = "NONE"
    OLDER_THAN_1_MONTH = "OLDER_THAN_1_MONTH"
    OLDER_THAN_6_MONTHS = "OLDER_THAN_6_MONTHS"
    OLDER_THAN_1_YEAR = "OLDER_THAN_1_YEAR"


class RunReviewTableSortField(str, Enum):
    CREATED_AT = "CREATED_AT"
    COMPLETED_TIME = "COMPLETED_TIME"
    STATUS = "STATUS"


class SortOrder(str, Enum):
    ASC = "ASC"
    DESC = "DESC"


class TestResultReviewSortField(str, Enum):
    STARTED_AT = "STARTED_AT"
    STATUS = "STATUS"
    COMPLETED_AT = "COMPLETED_AT"


class RunResultSortField(str, Enum):
    NAME = "NAME"
    STARTED_AT = "STARTED_AT"
    COMPLETED_AT = "COMPLETED_AT"
    PASS_PERCENTAGE = "PASS_PERCENTAGE"


class TestResultCheckErrorEnum(str, Enum):
    ALL = "ALL"
    SOME_CHECKS_FAILED = "SOME_CHECKS_FAILED"
    ALL_CHECKS_FAILED = "ALL_CHECKS_FAILED"
    ALL_CHECKS_PASSED = "ALL_CHECKS_PASSED"
    TESTS_WITH_OUTPUT_ERRORS = "TESTS_WITH_OUTPUT_ERRORS"
    SOME_LOW_CONFIDENCE_CHECKS = "SOME_LOW_CONFIDENCE_CHECKS"


class TestSuiteSortField(str, Enum):
    TITLE = "TITLE"
    CREATED = "CREATED"
    LAST_MODIFIED_AT = "LAST_MODIFIED_AT"


class CustomMetricReviewType(str, Enum):
    BASE = "BASE"
    RESULT_A = "RESULT_A"
    RESULT_B = "RESULT_B"
