# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List

from pydantic import Field

from .base_model import BaseModel


class GetOperators(BaseModel):
    operators: List["GetOperatorsOperators"]


class GetOperatorsOperators(BaseModel):
    name_in_doc: str = Field(alias="nameInDoc")
    is_unary: bool = Field(alias="isUnary")


GetOperators.model_rebuild()
