# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class EditCustomMetric(BaseModel):
    edit_custom_metric: Optional["EditCustomMetricEditCustomMetric"] = Field(
        alias="editCustomMetric"
    )


class EditCustomMetricEditCustomMetric(BaseModel):
    id: Optional[str]


EditCustomMetric.model_rebuild()
