# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class PollCustomMetricTask(BaseModel):
    poll_custom_metric_task: Optional["PollCustomMetricTaskPollCustomMetricTask"] = (
        Field(alias="pollCustomMetricTask")
    )


class PollCustomMetricTaskPollCustomMetricTask(BaseModel):
    status: str
    result: Optional[Any]
    error: Optional[str]
    traceback: Optional[str]


PollCustomMetricTask.model_rebuild()
