# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class AddAllTestsToQueueSingle(BaseModel):
    add_all_single_test_review_to_queue: Optional[
        "AddAllTestsToQueueSingleAddAllSingleTestReviewToQueue"
    ] = Field(alias="addAllSingleTestReviewToQueue")


class AddAllTestsToQueueSingleAddAllSingleTestReviewToQueue(BaseModel):
    single_run_review: (
        "AddAllTestsToQueueSingleAddAllSingleTestReviewToQueueSingleRunReview"
    ) = Field(alias="singleRunReview")


class AddAllTestsToQueueSingleAddAllSingleTestReviewToQueueSingleRunReview(BaseModel):
    id: Any


AddAllTestsToQueueSingle.model_rebuild()
AddAllTestsToQueueSingleAddAllSingleTestReviewToQueue.model_rebuild()
