# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List

from pydantic import Field

from .base_model import BaseModel


class GetUserOptions(BaseModel):
    user_emails: List[str] = Field(alias="userEmails")
