mutation AddOrRemoveUsersToRunReview(
  $singleRunReviewId: String!
  $projectId: String!
  $testResultReviewOptions: [TestResultReviewOptionsInput]!
) {
  updateAssignedReviewers(
    singleRunReviewId: $singleRunReviewId
    projectId: $projectId
    testResultReviewOptions: $testResultReviewOptions
  ) {
    newUsersAddedToQueue
  }
}
