# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class GetRunDataframe(BaseModel):
    get_run_dataframe: List["GetRunDataframeGetRunDataframe"] = Field(
        alias="getRunDataframe"
    )


class GetRunDataframeGetRunDataframe(BaseModel):
    id: str
    tags: List[str]
    input: str
    input_context: Any = Field(alias="inputContext")
    output: str
    output_context: Any = Field(alias="outputContext")
    right_answer: str = Field(alias="rightAnswer")
    refused_to_answer: bool = Field(alias="refusedToAnswer")
    is_rephrasal: bool = Field(alias="isRephrasal")
    been_rephrased: bool = Field(alias="beenRephrased")
    file_ids: List[str] = Field(alias="fileIds")
    operator: str
    criteria: str
    eval: int
    cont: float
    feedback: str
    is_global: bool = Field(alias="isGlobal")
    modifiers: Any


GetRunDataframe.model_rebuild()
