mutation startRun(
  $test_suite_id: String!
  $typed_parameters: ParameterInputType!
  $qa_set_id: String = null
  $run_name: String = null
  $run_id: String = null
) {
  startRun(
    testSuiteId: $test_suite_id
    typedParameters: $typed_parameters
    qaSetId: $qa_set_id
    runName: $run_name
    runId: $run_id
  ) {
    runId
  }
}

mutation updateRunStatus($run_id: String!, $status: RunStatus!) {
  updateRunStatus(runId: $run_id, status: $status) {
    run {
      runId
      status
    }
  }
}

mutation stopRun($run_id: String!, $status: RunStatus!) {
  stopRun(runId: $run_id, status: $status) {
    success
  }
}

mutation updateCustomMetricResult($runId: String!, $local: Boolean!, $custom_metric_result: Float!, $custom_metric_id: String, $custom_metric_name: String) {
  updateCustomMetricResult(runId: $runId, local: $local, customMetricResult: $custom_metric_result, customMetricId: $custom_metric_id, customMetricName: $custom_metric_name) {
    id
  }
}
