mutation AddAllTestsToQueueSingle(
  $runId: String!
  $templateIds: [String!]
  $testResultReviewOptions: [TestResultReviewOptionsInput]!
  $numberOfReviews: Int!
  $rereviewAutoEval: Boolean!
  $projectId: String!
) {
  addAllSingleTestReviewToQueue(
    runId: $runId
    templateIds: $templateIds
    testResultReviewOptions: $testResultReviewOptions
    numberOfReviews: $numberOfReviews
    rereviewAutoEval: $rereviewAutoEval
    projectId: $projectId
  ) {
    singleRunReview {
      id
    }
  }
}
