# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .fragments import TestFragment


class GetTestData(BaseModel):
    tests_with_count: "GetTestDataTestsWithCount" = Field(alias="testsWithCount")


class GetTestDataTestsWithCount(BaseModel):
    tests: List["GetTestDataTestsWithCountTests"]
    count: int


class GetTestDataTestsWithCountTests(TestFragment):
    pass


GetTestData.model_rebuild()
GetTestDataTestsWithCount.model_rebuild()
