#import TestFragment from "../fragments/test.graphql"
#import CheckOutputTypeFields from "../fragments/shared.graphql"

query getTestSuiteData($suiteId: String!) {
  testSuite(testSuiteId: $suiteId) {
    description
    id
    project {
      slug
    }
    org
    title
    created
    globalChecks {
      ...CheckOutputTypeFields
    }
  }
}

query getTestData($suiteId: String!, $offset: Int, $limit: Int) {
  testsWithCount(
    testSuiteId: $suiteId
    filterOptions: { offset: $offset, limit: $limit }
  ) {
    tests {
      ...TestFragment
    }
    count
  }
}

query getTestSuitesWithCount(
  $offset: Int
  $limit: Int
  $search: String
  $projectId: String!
) {
  testSuitesWithCount(
    filterOptions: {
      offset: $offset
      limit: $limit
      search: $search
      projectId: $projectId
    }
  ) {
    testSuites {
      id
      title
      description
      created
      creator
      lastModifiedBy
      lastModifiedAt
      folder {
        id
        name
      }
    }
  }
}

query GetDefaultParameters {
  defaultParameters {
    evalModel
    maximumThreads
    runConfidenceEvaluation
    createTextSummary
    detectRefusals
    modelUnderTest
    temperature
    maxOutputTokens
    heavyweightFactor
    systemPrompt
  }
}
