mutation CreateQuestionAnswerSet(
  $testSuiteId: String!
  $questionAnswerPairs: [QuestionAnswerPairInputType!]!
  $parameters: GenericScalar!
  $projectId: String!
  $modelId: String!
  $runName: String
) {
  createQuestionAnswerSet(
    testSuiteId: $testSuiteId
    questionAnswerPairs: $questionAnswerPairs
    parameters: $parameters
    projectId: $projectId
    modelId: $modelId
    runName: $runName
    createRun: true
  ) {
    questionAnswerSet {
      id
    }
    runId
  }
}

mutation BatchAddQuestionAnswerPairs(
  $questionAnswerSetId: String!
  $questionAnswerPairs: [QuestionAnswerPairInputType!]!
) {
  batchAddQuestionAnswerPairs(
    questionAnswerSetId: $questionAnswerSetId
    questionAnswerPairs: $questionAnswerPairs
  ) {
    questionAnswerPairs {
      id
      inputUnderTest
      llmOutput
      fileIds
      context
      outputContext
      test {
        id
      }
    }
  }
}

mutation MarkQuestionAnswerSetAsComplete($questionAnswerSetId: String!) {
  markQuestionAnswerSetAsComplete(questionAnswerSetId: $questionAnswerSetId) {
    questionAnswerSet {
      id
    }
  }
}
