query GetSingleRunReview($runReviewId: String!, $projectId: String!) {
  singleRunReview(runReviewId: $runReviewId, projectId: $projectId) {
    id
    createdBy
    createdAt
    status
    passRateHumanEval
    agreementRateAutoEval
    agreementRateHumanEval
    flaggedRate
    completedTime
    numberOfReviews
    assignedReviewers
    rereviewAutoEval
    project {
      id
    }
    run {
      id
    }
  }
}
