# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class ListProjects(BaseModel):
    projects_with_count: "ListProjectsProjectsWithCount" = Field(
        alias="projectsWithCount"
    )


class ListProjectsProjectsWithCount(BaseModel):
    projects: List["ListProjectsProjectsWithCountProjects"]
    count: int


class ListProjectsProjectsWithCountProjects(BaseModel):
    id: Any
    name: str
    slug: str
    created: datetime
    is_default: bool = Field(alias="isDefault")


ListProjects.model_rebuild()
ListProjectsProjectsWithCount.model_rebuild()
