# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class AddOrRemoveUsersToRunReview(BaseModel):
    update_assigned_reviewers: Optional[
        "AddOrRemoveUsersToRunReviewUpdateAssignedReviewers"
    ] = Field(alias="updateAssignedReviewers")


class AddOrRemoveUsersToRunReviewUpdateAssignedReviewers(BaseModel):
    new_users_added_to_queue: List[str] = Field(alias="newUsersAddedToQueue")


AddOrRemoveUsersToRunReview.model_rebuild()
