query ListQuestionAnswerPairs($qaSetId: String!, $offset: Int, $limit: Int) {
  questionAnswerPairsWithCount(
    qaSetId: $qaSetId
    filterOptions: { offset: $offset, limit: $limit }
  ) {
    questionAnswerPairs {
      id
      inputUnderTest
      llmOutput
      context
      outputContext
      metadata {
        inTokens
        outTokens
        durationSeconds
      }
      fileIds
      localEvals {
        id
        score
        feedback
        createdAt
      }
      test {
        id
      }
    }
    count
  }
}
